
const {
  isTargetExist,
} = require("@dao-style/vue-cli-plugin-shared-utils/lib/utils");
const stylelint = {
  pkg: {
    scripts: {
      "lint:style": "stylelint \"src/**/*.(vue|scss|css)\" --config .stylelintrc.js --fix",
    },
    "lint-staged": {
      '*.{vue,html,css,less,scss,sass}': 'npm run lint:style',
    },
    devDependencies: {
      // "stylelint": "^14.2.0",
      "stylelint": "^13.13.1",
      "stylelint-config-standard": "^22.0.0",
      "stylelint-config-sass-guidelines": "^8.0.0",
      "stylelint-config-recess-order": "^3.0.0",

    }
  },
  template: "./template/stylelint",
};
const eslint = {
  pkg: {
    scripts: {
      "lint:es": "eslint \"src/**/*.{ts,tsx,js,jsx,vue}\" --fix --max-warnings=0",
    },
    "lint-staged": {
      "*.{js,jsx,vue,ts,tsx}": ["npm run lint:es"]
    },
    devDependencies: {
      "@vue/eslint-config-airbnb": "^5.0.2",
      "eslint": "^7.32.0",
      "eslint-plugin-import": "^2.22.1",
      "eslint-plugin-vue": "^7.18.0",
      "eslint-import-resolver-alias": "^1.1.2",
      "eslint-import-resolver-node": "^0.3.6",
    }
  },
  template: "./template/eslint",
  genLint({ api }) {
    let pkg = this.pkg
    let template = this.template
    if (api.hasPlugin("typescript")) {
      pkg.devDependencies = Object.assign(pkg.devDependencies,{
        "@typescript-eslint/eslint-plugin": "^4.18.0",
        "@typescript-eslint/parser": "^4.18.0",
        "@vue/cli-plugin-typescript": "~4.5.0",
        "@vue/eslint-config-typescript": "^7.0.0",
        "vue-tsc": "^0.31.1"
      });
      pkg.scripts = Object.assign(pkg.scripts,{
        typecheck: "vue-tsc --noEmit",
      });
      pkg["lint-staged"]["*.{ts,tsx,vue}"] = () => 'vue-tsc --noEmit --project tsconfig.json';
      // pkg["lint-staged"]["*.{ts,tsx,vue}"] = ["npm run typecheck"];
      template = "./template/ts-eslint";
    }
    return { pkg,template,prepareStr: "" }
  },
};
const commitlint = {
  pkg: {
    scripts: {
      "commit": "git-cz",
    },
    config: {
      "commitizen": {
        "path": "@commitlint/cz-commitlint"
      }
    },
    devDependencies: {
      "@commitlint/cli": "^16.0.2",
      "@commitlint/config-conventional": "^16.0.0",
      "@commitlint/cz-commitlint": "^16.0.0",
      "commitizen": "^4.2.4",
    }
  },
  template: "./template/commitlint",
  genLint() {
    return { pkg: this.pkg,template: this.template,prepareStr: "npx husky add .husky/commit-msg 'npx commitlint --edit $1'" }
  },
};
function genLint({ lintInfo }) {
  return { pkg: lintInfo.pkg,template: lintInfo.template }
}
module.exports = (api,options,rootOptions) => {
  const renderList = ["./template/common"],
    prepareStrList = [],
    lintStagedObj = {};
  let hasLintStaged = false;
  // .设置husky
  if (!api.hasPlugin("husky")) {
    api.extendPackage({
      "devDependencies": {
        "husky": "^7.0.4"
      }
    })
  }
  if (!isTargetExist("./.husky")) {
    prepareStrList.push("husky install");
  }
  // .添加lint配置，pkg
  [stylelint,eslint,commitlint].forEach(item => {
    const { pkg,template,prepareStr } = item.genLint ? item.genLint({ api }) : genLint({ lintInfo: item });
    if (pkg) {
      let lintKeys = Object.keys(pkg["lint-staged"] || {})
      if (lintKeys.length) {
        Object.assign(lintStagedObj,pkg["lint-staged"])
        delete pkg["lint-staged"]
        if (!hasLintStaged) {
          prepareStrList.push("npx husky add .husky/pre-commit 'npx lint-staged --allow-empty $1'");
          hasLintStaged = true
        }
      }
      api.extendPackage(pkg);
    }
    template && renderList.push(template);
    prepareStr && prepareStrList.push(prepareStr);
  })
  // .inject vscode setting
  require("./injectVscode")();
  // .添加scripts.prepare
  prepareStrList.length && api.extendPackage({
    scripts: {
      prepare: prepareStrList.join(" && ")
    }
  });
  // .处理lint-staged文件，目前直接用文件覆盖
  // console.log(lintStagedObj)

  // .render
  renderList.forEach(file => {
    api.render(file)
  })
};
